clc;
clear;
close all;

total = 30;

annoPath = './annotation/';

numTrackers = 11;
resultPath = cell(numTrackers,1);
resultPath{1} = './result/SIFT/';
resultPath{2} = './result/FERNS/';
resultPath{3} = './result/SURF/';
resultPath{4} = './result/SOL/';
resultPath{5} = './result/SCV/';
resultPath{6} = './result/ESM/';
resultPath{7} = './result/IC/';
resultPath{8} = './result/GPF/';
resultPath{9} = './result/GO-ESM/';
resultPath{10} = './result/L1APG/';
resultPath{11} = './result/IVT/';

trackerNames = cell(numTrackers,1);
trackerNames{1} = 'SIFT';
trackerNames{2} = 'FERNS';
trackerNames{3} = 'SURF';
trackerNames{4} = 'SOL';
trackerNames{5} = 'SCV';
trackerNames{6} = 'ESM';
trackerNames{7} = 'IC';
trackerNames{8} = 'GPF';
trackerNames{9} = 'GO-ESM';
trackerNames{10} = 'L1APG';
trackerNames{11} = 'IVT';

extensionNames = cell(numTrackers,1);
extensionNames{1} = '_SIFT.txt';
extensionNames{2} = '_Ferns.txt';
extensionNames{3} = '_SURF.txt';
extensionNames{4} = '_SOL.txt';
extensionNames{5} = '_scv.txt';
extensionNames{6} = '_ssd.txt';
extensionNames{7} = '_IC.txt';
extensionNames{8} = '_gpf.txt';
extensionNames{9} = '_GO-ESM.txt';
extensionNames{10} = '_L1APG.txt';
extensionNames{11} = '_IVT.txt';

thresholdSet = 0:1:20;

flagSet = cell(total,7);
gtFilterSet = cell(total,7);
distSet = cell(numTrackers,1);

for seqId = 1:total
    for cJ = 1:7
        seqName =  sprintf('V%02d_%d',seqId,cJ);
        gtFile = [annoPath seqName '_gt_points.txt'];
        gt = load(gtFile);
        gt = gt(2:end,:);
        inx = 1:500;
        inx = find(mod(inx,2) ~= 0);
        gtFilter = gt(inx,:);
        gtFilterSet{seqId,cJ} = gtFilter;
        
        flagFile = [annoPath seqName '_flag.txt'];
        flag = load(flagFile);
        flag = flag(2:end,:);
        flagFilter = flag(inx,:);
        flagFilter = flagFilter == 0;
        flagSet{seqId,cJ} = flagFilter;
    end
end

for tId = 1:numTrackers
    distSet{tId} = cell(total,7);
    for seqI = 1:total
        for cJ = 1:7
            seqName =  sprintf('V%02d_%d',seqI,cJ);
            rtFile = [resultPath{tId} seqName extensionNames{tId}];
            rt = load(rtFile);
            rt = rt(2:end,:);   
            gtFilter = gtFilterSet{seqI,cJ};
            inx = 1:500;
            inx = find(mod(inx,2) ~= 0);
            rtFilter = rt(inx,:);
            aerr = gtFilter - rtFilter;
            aerr = aerr.^2;
            aerr = sum(aerr,2);
            aerr = aerr / 4;
            aerr = sqrt(aerr);
            distSet{tId}{seqI,cJ} = aerr;
        end
    end
end

preSet = cell(numTrackers,1);
for tId = 1:numTrackers
    numT = size(thresholdSet,2);
    pre = zeros(total,7,numT);
    for seqI = 1:total
        for cJ = 1:7
            flag = flagSet{seqI,cJ};
            curErr = distSet{tId}{seqI,cJ};
            for k = 1:numT
                curThreshold = thresholdSet(k);
                mask = curErr <= curThreshold;
                mask = mask.*flag;
                pre(seqI,cJ,k) = sum(mask)/sum(flag);
            end
        end
    end
    preSet{tId} = pre;
end

overallPlot = cell(numTrackers,1);
for tId = 1:numTrackers
    curPlot = zeros(1,numT);
    for k = 1:numT
        sumPre = 0;
        for seqI = 1:total
            for cJ = 1:7
                sumPre = sumPre+preSet{tId}(seqI,cJ,k);
            end
        end
        curPlot(k) = sumPre/(total*7);
    end
    overallPlot{tId} = curPlot;
end

colorValue = cell(11,1);
colorValue{1} = [1 0 0];
colorValue{2} = [0 0 1];
colorValue{3} = [0 1 0];
colorValue{4} = [163 73 164]/255;
colorValue{5} = [0 100 0]/255;
colorValue{6} = [255 127 39]/255;
colorValue{7} = [128 0 0]/255;
colorValue{8} = [128 128 0]/255;
colorValue{9} = [0 0 139]/255;
colorValue{10} = [128 0 128]/255;
colorValue{11} = [0 128 128]/255;

rankData = zeros(numTrackers,1);
legendNameSet = cell(numTrackers,1);
for tId = 1:numTrackers
    rankData(tId) = overallPlot{tId}(6);
    legendNameSet{tId} = sprintf('%s [%.4f]',trackerNames{tId},rankData(tId));
end

[sKey,inxKey] = sort(rankData,'descend');

figure;
box on;
hold on;

for tId = 1:numTrackers
    plot(thresholdSet,overallPlot{inxKey(tId)},'-','LineWidth',4,'Color',colorValue{inxKey(tId)});
end

legend(legendNameSet{inxKey(1)},legendNameSet{inxKey(2)},legendNameSet{inxKey(3)},legendNameSet{inxKey(4)},...
    legendNameSet{inxKey(5)},legendNameSet{inxKey(6)}, legendNameSet{inxKey(7)}, legendNameSet{inxKey(8)},...
    legendNameSet{inxKey(9)}, legendNameSet{inxKey(10)}, legendNameSet{inxKey(11)}, 'Location','SouthEast');
set(gcf,'color','white');
set(gca,'FontSize',30);
set(gcf,'Position',[10 10 1200 960]);
title('All sequences','FontSize',40);
xlabel('Alignment error threshold','FontSize',40);
ylabel('Precision','FontSize',40);
imwrite(frame2im(getframe(gcf)),'All-seq-precision.png');

fprintf('finished...\n');