clc;
clear;
close all;

total = 30;

annoPath = './annotation/';

numTrackers = 11;
resultPath = cell(numTrackers,1);
resultPath{1} = './result/SIFTHomography/';
resultPath{2} = './result/FERNSHomography/';
resultPath{3} = './result/SURFHomography/';
resultPath{4} = './result/SOLHomography/';
resultPath{5} = './result/SCVHomography/';
resultPath{6} = './result/ESMHomography/';
resultPath{7} = './result/ICHomography/';
resultPath{8} = './result/GPFHomography/';
resultPath{9} = './result/GO-ESMHomography/';
resultPath{10} = './result/L1APGHomography/';
resultPath{11} = './result/IVTHomography/';


trackerNames = cell(numTrackers,1);
trackerNames{1} = 'SIFT';
trackerNames{2} = 'FERNS';
trackerNames{3} = 'SURF';
trackerNames{4} = 'SOL';
trackerNames{5} = 'SCV';
trackerNames{6} = 'ESM';
trackerNames{7} = 'IC';
trackerNames{8} = 'GPF';
trackerNames{9} = 'GO-ESM';
trackerNames{10} = 'L1APG';
trackerNames{11} = 'IVT';

extensionNames = cell(numTrackers,1);
extensionNames{1} = '_SIFT_homography.txt';
extensionNames{2} = '_Ferns_homography.txt';
extensionNames{3} = '_SURF_homography.txt';
extensionNames{4} = '_SOL_homography.txt';
extensionNames{5} = '_scv_homography.txt';
extensionNames{6} = '_ssd_homography.txt';
extensionNames{7} = '_IC_homography.txt';
extensionNames{8} = '_gpf_homography.txt';
extensionNames{9} = '_GO-ESM_homography.txt';
extensionNames{10} = '_L1APG_homography.txt';
extensionNames{11} = '_IVT_homography.txt';

flagSet = cell(total,7);
gtFilterSet = cell(total,7);

for seqId = 1:total
    for cJ = 1:7
        seqName =  sprintf('V%02d_%d',seqId,cJ);
        gtFile = [annoPath seqName '_gt_homography.txt'];
        gt = load(gtFile);
        gt = gt(2:end,:);
        inx = 1:500;
        inx = find(mod(inx,2) ~= 0);
        gtFilter = gt(inx,:);
        gtFilterSet{seqId,cJ} = gtFilter;
        
        flagFile = [annoPath seqName '_flag.txt'];
        flag = load(flagFile);
        flag = flag(2:end,:);
        flagFilter = flag(inx,:);
        flagFilter = flagFilter == 0;
        flagSet{seqId,cJ} = flagFilter;
    end
end

corners = [-1 -1 1;1 -1 1;-1 1 1;1 1 1];

homoSet = cell(numTrackers,1);

for tId = 1:numTrackers
    homoSet{tId} = cell(total,7);
    for seqI = 1:total
        for cJ = 1:7
            seqName =  sprintf('V%02d_%d',seqI,cJ);
            gtFilter =  gtFilterSet{seqI,cJ};
          
            rtFile = [resultPath{tId} seqName extensionNames{tId}];
            rt = load(rtFile);
            rt = rt(2:end,:);
            inx = 1:500;
            inx = find(mod(inx,2) ~= 0);
            rtFilter = rt(inx,:);
            
            N = size(gtFilter,1);
            heer = zeros(N,1);
            for i = 1:N
                T_gt = [gtFilter(i,1) gtFilter(i,2) gtFilter(i,3);gtFilter(i,4) gtFilter(i,5) gtFilter(i,6);...
                    gtFilter(i,7) gtFilter(i,8) gtFilter(i,9)];
                T_rt = [rtFilter(i,1) rtFilter(i,2) rtFilter(i,3);rtFilter(i,4) rtFilter(i,5) rtFilter(i,6);...
                    rtFilter(i,7) rtFilter(i,8) rtFilter(i,9)];
                distance = 0;
                for k = 1:4
                    H = T_gt*pinv(T_rt);
                    pt = corners(k,:)';
                    ptProj = H*pt;
                    ptProj(1) = ptProj(1)/ptProj(3);
                    ptProj(2) = ptProj(2)/ptProj(3);
                    dx = pt(1)-ptProj(1);
                    dy = pt(2)-ptProj(2);
                    distance = distance+sqrt(dx*dx+dy*dy)/4;
                end
                heer(i) = distance;
            end
            homoSet{tId}{seqI,cJ} = heer;
        end
    end
end

thresholdSet = 0:10:200;
numT = size(thresholdSet,2);

preSet = cell(numTrackers,1);
for tId = 1:numTrackers
    pre = zeros(total,7,numT);
    for seqId = 1:total
        for cJ = 1:7
            flag = flagSet{seqId,cJ};
            curErr = homoSet{tId}{seqId,cJ};
            for k = 1:numT
                curThreshold = thresholdSet(k);
                mask = curErr <= curThreshold;
                mask = mask.*flag;
                pre(seqId,cJ,k) = sum(mask)/sum(flag);
            end
        end
    end
    preSet{tId} = pre;
end

overallPlot = cell(numTrackers,1);
for tId = 1:numTrackers
    curPlot = zeros(1,numT);
    for k = 1:numT
        sumPre = 0;
        for seqId = 1:total
            for cJ = 1:7
                sumPre = sumPre+preSet{tId}(seqId,cJ,k);
            end
        end
        curPlot(k) = sumPre/(total*7);
    end
    overallPlot{tId} = curPlot;
end

colorValue = cell(7,1);
colorValue{1} = [1 0 0];
colorValue{2} = [0 0 1];
colorValue{3} = [0 1 0];
colorValue{4} = [163 73 164]/255;
colorValue{5} = [0 100 0]/255;
colorValue{6} = [255 127 39]/255;
colorValue{7} = [128 0 0]/255;
colorValue{8} = [128 128 0]/255;
colorValue{9} = [0 0 139]/255;
colorValue{10} = [128 0 128]/255;
colorValue{11} = [0 128 128]/255;

rankData = zeros(numTrackers,1);
legendNameSet = cell(numTrackers,1);
for tId = 1:numTrackers
    rankData(tId) = overallPlot{tId}(2);
    legendNameSet{tId} = sprintf('%s [%.4f]',trackerNames{tId},rankData(tId));
end

[sKey,inxKey] = sort(rankData,'descend');

figure;
box on;
hold on;

for tId = 1:numTrackers
    plot(thresholdSet,overallPlot{inxKey(tId)},'-','LineWidth',4,'Color',colorValue{inxKey(tId)});
end

legend(legendNameSet{inxKey(1)},legendNameSet{inxKey(2)},legendNameSet{inxKey(3)},legendNameSet{inxKey(4)},...
    legendNameSet{inxKey(5)},legendNameSet{inxKey(6)}, legendNameSet{inxKey(7)}, legendNameSet{inxKey(8)},...
    legendNameSet{inxKey(9)}, legendNameSet{inxKey(10)}, legendNameSet{inxKey(11)}, 'Location','SouthEast');
set(gcf,'color','white');
set(gca,'FontSize',30);
set(gcf,'Position',[10 10 1200 960]);
title('All sequences','FontSize',40);
xlabel('Homography discrepancy threshold','FontSize',40);
ylabel('Success rate','FontSize',40);
imwrite(frame2im(getframe(gcf)),'All-seq-homography.png');

fprintf('finished...\n');