clc;
clear;
close all;

total = 30;

annoPath = './annotation/';

numTrackers = 11;
resultPath = cell(numTrackers,1);
resultPath{1} = './result/SIFT/';
resultPath{2} = './result/FERNS/';
resultPath{3} = './result/SURF/';
resultPath{4} = './result/SOL/';
resultPath{5} = './result/SCV/';
resultPath{6} = './result/ESM/';
resultPath{7} = './result/IC/';
resultPath{8} = './result/GPF/';
resultPath{9} = './result/GO-ESM/';
resultPath{10} = './result/L1APG/';
resultPath{11} = './result/IVT/';

trackerNames = cell(numTrackers,1);
trackerNames{1} = 'SIFT';
trackerNames{2} = 'FERNS';
trackerNames{3} = 'SURF';
trackerNames{4} = 'SOL';
trackerNames{5} = 'SCV';
trackerNames{6} = 'ESM';
trackerNames{7} = 'IC';
trackerNames{8} = 'GPF';
trackerNames{9} = 'GO-ESM';
trackerNames{10} = 'L1APG';
trackerNames{11} = 'IVT';

extensionNames = cell(numTrackers,1);
extensionNames{1} = '_SIFT.txt';
extensionNames{2} = '_Ferns.txt';
extensionNames{3} = '_SURF.txt';
extensionNames{4} = '_SOL.txt';
extensionNames{5} = '_scv.txt';
extensionNames{6} = '_ssd.txt';
extensionNames{7} = '_IC.txt';
extensionNames{8} = '_gpf.txt';
extensionNames{9} = '_GO-ESM.txt';
extensionNames{10} = '_L1APG.txt';
extensionNames{11} = '_IVT.txt';

thresholdSet = 0:1:20;

flagSet = cell(total,7);
gtFilterSet = cell(total,7);
distSet = cell(numTrackers,1);

for seqId = 1:total
    for cJ = 1:7
        seqName =  sprintf('V%02d_%d',seqId,cJ);
        gtFile = [annoPath  seqName '_gt_points.txt'];
        gt = load(gtFile);
        gt = gt(2:end,:);
        inx = 1:500;
        inx = find(mod(inx,2) ~= 0);
        gtFilter = gt(inx,:);
        gtFilterSet{seqId,cJ} = gtFilter;
        
        flagFile = [annoPath seqName '_flag.txt'];
        flag = load(flagFile);
        flag = flag(2:end,:);
        flagFilter = flag(inx,:);
        flagFilter = flagFilter == 0;
        flagSet{seqId,cJ} = flagFilter;
    end
end

for tId = 1:numTrackers
    distSet{tId} = cell(total,7);
    for seqId = 1:total
        for cJ = 1:7
            seqName =  sprintf('V%02d_%d',seqId,cJ);
            rtFile = [resultPath{tId} seqName extensionNames{tId}];
            rt = load(rtFile);
            rt = rt(2:end,:);   
            gtFilter = gtFilterSet{seqId,cJ};
            inx = 1:500;
            inx = find(mod(inx,2) ~= 0);
            rtFilter = rt(inx,:);
            aerr = gtFilter - rtFilter;
            aerr = aerr.^2;
            aerr = sum(aerr,2);
            aerr = aerr / 4;
            aerr = sqrt(aerr);
            distSet{tId}{seqId,cJ} = aerr;
        end
    end
end

colorValue = cell(11,1);
colorValue{1} = [1 0 0];
colorValue{2} = [0 0 1];
colorValue{3} = [0 1 0];
colorValue{4} = [163 73 164]/255;
colorValue{5} = [0 100 0]/255;
colorValue{6} = [255 127 39]/255;
colorValue{7} = [128 0 0]/255;
colorValue{8} = [128 128 0]/255;
colorValue{9} = [0 0 139]/255;
colorValue{10} = [128 0 128]/255;
colorValue{11} = [0 128 128]/255;

motionSet = cell(7,1);
motionSet{1} = 'SC';
motionSet{2} = 'RT';
motionSet{3} = 'PD';
motionSet{4} = 'MB';
motionSet{5} = 'OCC';
motionSet{6} = 'OV';
motionSet{7} = 'UC';

motionAllSet = cell(7,1);
motionAllSet{1} = 'Scale change';
motionAllSet{2} = 'Rotation';
motionAllSet{3} = 'Perspective distortion';
motionAllSet{4} = 'Motion blur';
motionAllSet{5} = 'Occlusion';
motionAllSet{6} = 'Out of view';
motionAllSet{7} = 'Unconstrained';

plotDataSet = cell(numTrackers,1);
rankSet = cell(numTrackers,1);

for tId = 1:numTrackers
    numT = size(thresholdSet,2);
    pre = zeros(total,7,numT);
    for seqI = 1:total
        for cJ = 1:7
            flag = flagSet{seqI,cJ};
            curErr = distSet{tId}{seqI,cJ};
            for k = 1:numT
                curThreshold = thresholdSet(k);
                mask = curErr <= curThreshold;
                mask = mask.*flag;
                pre(seqI,cJ,k) = sum(mask)/sum(flag);
            end
        end
    end
    plotData = zeros(7,numT);
    for cJ = 1:7
        for k = 1:numT
            sumPre = 0;
            for tpi =1:total;
                sumPre = sumPre+pre(tpi,cJ,k);
            end
            plotData(cJ,k) = sumPre/total;
        end
    end
    plotDataSet{tId} = plotData;
    rank = plotData(1:7,6);
    rankSet{tId} = rank;
end

legendNameSet = cell(numTrackers,7);
for tId = 1:numTrackers
    for i = 1:7
         strPre=sprintf(' [%.4f]',rankSet{tId}(i));
         legendNameSet{tId,i} = strcat(trackerNames{tId},strPre);
    end
end

for i = 1:7
    figure;
    box on;
    hold on;
    curRank = zeros(numTrackers,1);
    for tId = 1:numTrackers
        curRank(tId) = rankSet{tId}(i);
    end
    [sKey,inxKey] = sort(curRank,'descend');
     for tId = 1:numTrackers
        plot(thresholdSet,plotDataSet{inxKey(tId)}(i,:),'-','LineWidth',4,'Color',colorValue{inxKey(tId)});
    end 
    legend(legendNameSet{inxKey(1),i},legendNameSet{inxKey(2),i},legendNameSet{inxKey(3),i},legendNameSet{inxKey(4),i},legendNameSet{inxKey(5),i},... 
        legendNameSet{inxKey(6),i}, legendNameSet{inxKey(7),i}, legendNameSet{inxKey(8),i}, legendNameSet{inxKey(9),i},...
        legendNameSet{inxKey(10),i}, legendNameSet{inxKey(11),i}, 'Location','SouthEast');
    set(gcf,'color','white');
    set(gca,'FontSize',30);
    set(gcf,'Position',[10 10 1200 960]);
    title(motionAllSet{i},'FontSize',40);
    xlabel('Alignment error threshold','FontSize',40);
    ylabel('Precision','FontSize',40);
    saveFileName = [motionSet{i} '-precision.png'];
    imwrite(frame2im(getframe(gcf)),saveFileName);   
end

numKeypoint = 4;
keypointTrackerIndex = [1,2,3,4];

numRegion = 5;
regionTrackerIndex = [5,6,7,8,9];

numT = size(thresholdSet,2);
keypointPlotData = zeros(7,numT);
regionPlotData = zeros(7,numT);

for i = 1:numKeypoint
    keypointPlotData = keypointPlotData + plotDataSet{keypointTrackerIndex(i)};
end
keypointPlotData = keypointPlotData./numKeypoint;

for i = 1:numRegion
    regionPlotData = regionPlotData + plotDataSet{regionTrackerIndex(i)};
end
regionPlotData = regionPlotData./numRegion;

for tpI = 1:2
    if tpI == 1
        plotData = keypointPlotData;
        titleName = 'Keypoint-based';
    else
        plotData = regionPlotData;
        titleName = 'Region-based';
    end
    rank = plotData(1:7,6);
    rankSet{tId} = rank;
    [sKey,inxKey] = sort(rank,'descend');
    
    legendName = cell(7,1);
    for i =1:7
        strPre=sprintf(' [%.4f]',rank(i));
        legendName{i} = strcat(motionSet{i},strPre);
    end
    
    figure;
    box on;
    hold on;
    
    plot(thresholdSet,plotData(inxKey(1),:),'-','LineWidth',4,'Color',colorValue{inxKey(1)});
    plot(thresholdSet,plotData(inxKey(2),:),'-','LineWidth',4,'Color',colorValue{inxKey(2)});
    plot(thresholdSet,plotData(inxKey(3),:),'-','LineWidth',4,'Color',colorValue{inxKey(3)});
    plot(thresholdSet,plotData(inxKey(4),:),'-','LineWidth',4,'Color',colorValue{inxKey(4)});
    plot(thresholdSet,plotData(inxKey(5),:),'-','LineWidth',4,'Color',colorValue{inxKey(5)});
    plot(thresholdSet,plotData(inxKey(6),:),'-','LineWidth',4,'Color',colorValue{inxKey(6)});
    plot(thresholdSet,plotData(inxKey(7),:),'-','LineWidth',4,'Color',colorValue{inxKey(7)});
    
    legend(legendName{inxKey(1)},legendName{inxKey(2)},legendName{inxKey(3)},legendName{inxKey(4)},...
        legendName{inxKey(5)},legendName{inxKey(6)},legendName{inxKey(7)},'Location','SouthEast');
    set(gcf,'color','white');
    set(gca,'FontSize',30);
    set(gcf,'Position',[10 10 1200 960]);
    title(titleName,'FontSize',40);
    xlabel('Alignment error threshold','FontSize',40);
    ylabel('Precision','FontSize',40);
    fileName = [titleName '.png'];
    imwrite(frame2im(getframe(gcf)),fileName);
end
fprintf('finished...\n');




